#if !defined(__TIMER_H)
    #define __TIMER_H
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Program name    : timer.h                                                  */
/*                                                                            */
/* Description     : This header file contains function prototypes and        */
/*                   constants for the timer plugin source files              */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: July 2001                    */
/*                                                                            */
/******************************************************************************/

#if defined(WIN32)
	#include <windows.h>
	#include <winbase.h>
#endif

#include <wchar.h>
#include <BipCci.h>	  /* Data types for Plug-in Node functions     */

#include <trace_publ.h>
#include "plugin.h"

#if defined(__THREADED)
    #include "threads.h"
#endif

#include "shmem.h"

#include <cmqc.h>
#include "unicode.h"
#include "utilities.h"
#include "MQSIV2_timer_msg.h"
#include "rc.h"
#include "timer_constants.h"

#if defined(_AIX) || defined(_SOLARIS) || defined(_HPUX)
    #include <values.h>
    #define INFINITE MAXINT
#endif

/* constants */
#define MAX_TIMER_NAME_LEN         256
#if !defined(IN)
    #define IN
#endif
#if !defined(OUT)
    #define OUT
#endif
#if !defined(INOUT)
    #define INOUT
#endif
#if !defined(TRUE)
    #define TRUE 1
#endif
#if !defined(FALSE)
    #define FALSE 0
#endif
#if !defined(bool_t)
    #define bool_t int
#endif

#define SHARED_MEMORY_SEGMENT_NAME_PREFIX "Timer_ShMem"
#define SHARED_SEGMENT_DEFAULT_KSIZE 64
#define SHARED_SEGMENT_KEY "POST"
#define SHARED_SEGMENT_PLATFORM_SCOPE_KEY "Timer_Platform"
#define SHARED_SEGMENT_BROKER_SCOPE_KEY_PREFIX "Timer_"

#if !defined(SLEEP_THREAD_MILLISECONDS)
	#if defined(WIN32)
		#define SLEEP_THREAD_MILLISECONDS(__interval) Sleep(__interval)
	#else
		#define SLEEP_THREAD_MILLISECONDS(__interval) usleep(__interval*1000)
	#endif
#endif

#define TIMER_DATA_BEGIN_TAG "<TimerEventData>"
#define TIMER_DATA_END_TAG "</TimerEventData>"
#define TIMER_DATA_NULL_TAG "<TimerEventData/>"


/* return sequence numbers */
#define RSN_001     1
#define RSN_002     2
#define RSN_003     3
#define RSN_004     4
#define RSN_005     5
#define RSN_006     6
#define RSN_007     7
#define RSN_008     8
#define RSN_009     9
#define RSN_010     10
#define RSN_011     11
#define RSN_012     12
#define RSN_013     13
#define RSN_014     14
#define RSN_015     15
#define RSN_016     16
#define RSN_017     17
#define RSN_018     18
#define RSN_019     19
#define RSN_020     20
#define RSN_021     21
#define RSN_022     22
#define RSN_023     23
#define RSN_024     24
#define RSN_025     25
#define RSN_026     26
#define RSN_027     27
#define RSN_028     28
#define RSN_029     29
#define RSN_030     30
#define RSN_031     31
#define RSN_032     32
#define RSN_033     33
#define RSN_034     34
#define RSN_035     35
#define RSN_036     36
#define RSN_037     37
#define RSN_038     38
#define RSN_039     39
#define RSN_040     40
#define RSN_041     41
#define RSN_042     42
#define RSN_043     43
#define RSN_044     44
#define RSN_045     45
#define RSN_046     46
#define RSN_047     47
#define RSN_048     48
#define RSN_049     49
#define RSN_050     50

/* typedefs */
#include <time.h>

typedef enum
{
	SCOPE_EXECUTION_GROUP = 1,
	SCOPE_BROKER,
	SCOPE_PLATFORM
} scope_t;

typedef struct timerInstance {
	char msgId[MQ_MSG_ID_LENGTH],
		 correlId[MQ_CORREL_ID_LENGTH];
	char *timerName;
	time_t timeoutTime;
	scope_t scope;
	bool_t expired,
		   onDeadChain,
		   global;
	void *savedDataP;
	int intervalSeconds,
		savedDataLength,
		timerPopCount,
		msgExpiry;
	struct deployedTimer *deployedTimerP;
	struct timerInstance *nextP,             /* deployedTimerNode ownership chain */
		                 *prevP,             /* deployedTimerNode ownership chain */
						 *nextMsgIdP,
						 *prevMsgIdP,
						 *nextCorrelIdP,
						 *prevCorrelIdP,
						 *nextNameP,
						 *prevNameP,
						 *nextBothP,
						 *prevBothP,
						 *nextAlarmP,
						 *prevAlarmP,
						 *nextGlobalP,
						 *prevGlobalP,
						 *nextGlobalMsgIdP,
						 *prevGlobalMsgIdP,
						 *nextGlobalCorrelIdP,
						 *prevGlobalCorrelIdP,
						 *nextGlobalBothP,
						 *prevGlobalBothP;
} timerInstance_t;
#define TIMER_INSTANCE_DEFAULT \
	                   NULL_24,      /* msgId */            \
	                   NULL_24,      /* correlId */         \
	                   NULL,    /* timerName */             \
	                   0,       /* timeoutTime */           \
					   SCOPE_EXECUTION_GROUP, /* scope */   \
					   FALSE,   /* expired */               \
					   FALSE,   /* onDeadChain */           \
					   FALSE,   /* global */                \
					   NULL,    /* savedDataP */            \
					   0,       /* intervalSeconds */       \
					   0,       /* savedDataLength */       \
					   0,       /* timerPopCount */         \
					   0,       /* msgExpiry */             \
	                   NULL,    /* deployedTimerP */        \
					   NULL,    /* nextP */                 \
					   NULL,    /* prevP */                 \
					   NULL,    /* nextMsgIdP */            \
					   NULL,    /* prevMsgIdP */            \
					   NULL,    /* nextCorrelIdP */         \
					   NULL,    /* prevCorrelIdP */         \
					   NULL,    /* nextNameP */             \
					   NULL,    /* prevNameP */             \
					   NULL,    /* nextBothP */             \
					   NULL,    /* prevBothP */             \
					   NULL,    /* nextAlarmP */            \
					   NULL,    /* prevAlarmP */            \
					   NULL,    /* nextGlobalP */           \
					   NULL,    /* prevGlobalP */           \
					   NULL,    /* nextGlobalMsgIdP */      \
					   NULL,    /* prevGlobalMsgIdP */      \
					   NULL,    /* nextGlobalCorrelIdP */   \
					   NULL,    /* prevGlobalCorrelIdP */   \
					   NULL,    /* nextGlobalBothP */       \
					   NULL     /* prevGlobalBothP */       

typedef struct globalTimerInstance {
	char msgId[MQ_MSG_ID_LENGTH],
		 correlId[MQ_CORREL_ID_LENGTH];
	shmemHandle_t timerName;
	time_t timeoutTime;
	scope_t scope;
	bool_t expired,
		   onDeadChain,
		   global;
	shmemHandle_t savedDataP;
	int intervalSeconds,
    	savedDataLength,
		timerPopCount,
		msgExpiry;
	struct deployedTimer *deployedTimerP;
	struct timerInstance *nextP,             /* deployedTimerNode ownership chain */
		                 *prevP,             /* deployedTimerNode ownership chain */
						 *nextMsgIdP,
						 *prevMsgIdP,
						 *nextCorrelIdP,
						 *prevCorrelIdP,
						 *nextNameP,
						 *prevNameP,
						 *nextBothP,
						 *prevBothP,
						 *nextAlarmP,
						 *prevAlarmP;
	shmemHandle_t		 nextGlobalP,
						 prevGlobalP,
						 nextGlobalMsgIdP,
						 prevGlobalMsgIdP,
						 nextGlobalCorrelIdP,
						 prevGlobalCorrelIdP,
						 nextGlobalNameP,
						 prevGlobalNameP,
						 nextGlobalBothP,
						 prevGlobalBothP;
	int instanceLength,
	    executionGroupPID;
	bool_t retrieveMessageDataFromTree;
	shmemHandle_t executionGroupNameP,
	              messageFlowNameP,
				  queueNameP;
} globalTimerInstance_t;
#define GLOBAL_TIMER_INSTANCE_DEFAULT \
	                   NULL_24,      /* msgId */            \
	                   NULL_24,      /* correlId */         \
	                   SHM_NULL_HANDLE, /* timerName */     \
	                   0,       /* timeoutTime */           \
					   SCOPE_EXECUTION_GROUP, /* scope */   \
					   FALSE,   /* expired */               \
					   FALSE,   /* onDeadChain */           \
					   FALSE,   /* global */                \
					   SHM_NULL_HANDLE,    /* savedDataP */ \
					   0,       /* intervalSeconds */       \
					   0,       /* savedDataLength */       \
					   0,       /* timerPopCount */         \
					   0,       /* msgExpiry */             \
	                   NULL,    /* deployedTimerP */        \
					   NULL,    /* nextP */                 \
					   NULL,    /* prevP */                 \
					   NULL,    /* nextMsgIdP */            \
					   NULL,    /* prevMsgIdP */            \
					   NULL,    /* nextCorrelIdP */         \
					   NULL,    /* prevCorrelIdP */         \
					   NULL,    /* nextNameP */             \
					   NULL,    /* prevNameP */             \
					   NULL,    /* nextBothP */             \
					   NULL,    /* prevBothP */             \
					   NULL,    /* nextAlarmP */            \
					   NULL,    /* prevAlarmP */            \
					   SHM_NULL_HANDLE, /* nextGlobalP */           \
					   SHM_NULL_HANDLE, /* prevGlobalP */           \
					   SHM_NULL_HANDLE, /* nextGlobalMsgIdP */      \
					   SHM_NULL_HANDLE, /* prevGlobalMsgIdP */      \
					   SHM_NULL_HANDLE, /* nextGlobalCorrelIdP */   \
					   SHM_NULL_HANDLE, /* prevGlobalCorrelIdP */   \
					   SHM_NULL_HANDLE, /* nextGlobalNameP */       \
					   SHM_NULL_HANDLE, /* prevGlobalNameP */       \
					   SHM_NULL_HANDLE, /* nextGlobalBothP */       \
					   SHM_NULL_HANDLE, /* prevGlobalBothP */       \
					   0,               /* instanceLength */        \
					   0,               /* executionGroupPID */     \
					   FALSE,   /* retrieveMessageDataFromTree */   \
					   SHM_NULL_HANDLE, /* executionGroupNameP */   \
					   SHM_NULL_HANDLE, /* messageFlowNameP */      \
					   SHM_NULL_HANDLE  /* queueNameP */ 

typedef enum
{
	MESSAGE_TYPE_BLOB = 1,
	MESSAGE_TYPE_XML
} messageType_t;

typedef struct deployedTimer {
	CciChar *timerName,
	        *messageData,
			*intervalSecondsElement;
	CciChar queueName[MQ_Q_NAME_LENGTH + 1];
	bool_t retrieveMessageDataFromTree,
	       retrieveTimerNameFromTree,
		   retrieveIntervalSecondsFromTree;
	messageType_t messageType;
	int intervalSeconds,
	    maxTimerPopCount,
		msgExpiry;
	scope_t scope;
    timerInstance_t instanceAnchor;
	struct deployedTimer *nextP,
		                 *prevP;
} deployedTimer_t;
#define DEPLOYED_TIMER_DEFAULT \
	                   NULL,  /* timerName */   \
	                   NULL,  /* messageData */  \
	                   NULL,  /* intervalSecondsElement */  \
					   NULL_48_AND_TRAILING_NULL,   /* queueName */ \
					   FALSE, /* retrieveMessageDataFromTree */ \
					   FALSE, /* retrieveTimerNameFromTree */ \
					   FALSE, /* retrieveIntervalSecondsFromTree */ \
                       MESSAGE_TYPE_BLOB, /* messageType */ \
                       0,     /* intervalSeconds */  \
	                   0,     /* maxTimerPopCount */ \
	                   0,     /* msgExpiry */ \
					   SCOPE_EXECUTION_GROUP, /* scope */ \
                       {TIMER_INSTANCE_DEFAULT}, \
	                   NULL,  /* nextP */ \
					   NULL   /* prevP */

typedef enum
{
	MATCH_TYPE_MSGID_MSGID = 1,
	MATCH_TYPE_MSGID_CORRELID,
	MATCH_TYPE_CORRELID_CORRELID,
	MATCH_TYPE_CORRELID_MSGID,
	MATCH_TYPE_BOTH
} msgMatchType_t;

typedef enum
{
	INDEX_MSGID = 1,
	INDEX_CORRELID,
	INDEX_BOTH,
	INDEX_NAME
} indexType_t;

typedef struct deployedTimerCancel {
	CciChar *timerName;
	bool_t retrieveTimerNameFromTree;
	msgMatchType_t matchValue;
	scope_t scope;
	struct deployedTimerCancel *nextP,
		                       *prevP;
} deployedTimerCancel_t;
#define DEPLOYED_TIMER_CANCEL_DEFAULT \
	                   NULL,  /* timerName */  \
                       FALSE, /* retrieveTimerNameFromTree */  \
                       0,     /* matchValue */  \
					   SCOPE_EXECUTION_GROUP, /* scope */ \
	                   NULL,  /* nextP */ \
					   NULL   /* prevP */

typedef enum
{
	TIMER_CREATE = 1,
	TIMER_CANCEL
} nodeType_t;

typedef struct executionGroupEntry {
    shmemHandle_t name;
	int pid;
	shmemHandle_t nextEntry,
		          prevEntry;
} executionGroupEntry_t;
#define EXECUTION_GROUP_ENTRY_DEFAULT  \
    SHM_NULL_HANDLE,        /* name */ \
    0,                      /* pid */ \
    SHM_NULL_HANDLE,        /* nextEntry */ \
    SHM_NULL_HANDLE         /* prevEntry */

typedef struct globalCB {
	int CBLength;
    globalTimerInstance_t globalInstanceAnchor,
                          globalDeadInstanceAnchor;
	executionGroupEntry_t executionGroupAnchor;
} globalCB_t;
#define GLOBAL_CB_DEFAULT \
	                   sizeof(globalCB_t),            /* CBLength */ \
	                   GLOBAL_TIMER_INSTANCE_DEFAULT, /* globalInstanceAnchor */ \
	                   GLOBAL_TIMER_INSTANCE_DEFAULT, /* deadInstanceAnchor */   \
	                   EXECUTION_GROUP_ENTRY_DEFAULT  /* executionGroupAnchor */  

extern char *traceRetcodeStrings[];
extern int traceMaxRetcodeValue;

#if defined(GLOBAL_TIMER_DATA)
	/* global variables */
    threadMutex_t timerChainMutex,
				  alarmThreadSyncMutex,
				  shmemThreadSyncMutex,
				  shmemThreadWorkMutex,
                  shmemInitTermMutex;
	threadCondVar_t alarmThreadWorkCondVar,
		            alarmThreadSyncCondVar,
					shmemThreadWorkCondVar,
					shmemThreadSyncCondVar;
	bool_t shutdownAlarmRequested = FALSE,
	       shutdownShmemRequested = FALSE,
		   alarmThreadInitialised = FALSE,
		   alarmThreadCreated = FALSE,
		   shmemThreadInitialised = FALSE,
		   shmemThreadCreated = FALSE,
		   brokerInfoObtained = FALSE,
		   alarmThreadShuttingDown = FALSE,
		   shmemThreadShuttingDown = FALSE,
		   firstTraceCall = TRUE,
		   sharedMemoryAttached_Broker = FALSE,
		   sharedMemoryAttached_Platform = FALSE,
#if defined(WIN32)
		   platformIsUnix = FALSE,
#else
		   platformIsUnix = TRUE,
#endif
		   sharedMemoryGarbageCollectorHasRun = FALSE,
		   sharedMemoryExceptionThrown = FALSE;

	deployedTimer_t timerAnchor = {DEPLOYED_TIMER_DEFAULT},
		           *timerAnchorP = &timerAnchor;
	deployedTimerCancel_t timerCancelAnchor = {DEPLOYED_TIMER_CANCEL_DEFAULT},
		                  *timerCancelAnchorP = &timerCancelAnchor;
	
	timerInstance_t instanceAnchor = {TIMER_INSTANCE_DEFAULT},
		                   *nameIndexAnchorP = &instanceAnchor,
		                   *msgIdIndexAnchorP = &instanceAnchor,
						   *correlIdIndexAnchorP = &instanceAnchor,
						   *bothIndexAnchorP = &instanceAnchor;

	messageFlow_t messageFlowAnchor = {MESSAGE_FLOW_DEFAULT};

	char *alarmThreadErrorMessage = NULL,
	     *shmemThreadErrorMessage = NULL;
	threadHandle_t alarmThreadHandle,
		           shmemThreadHandle;
	
	char queueManagerName[MQ_Q_MGR_NAME_LENGTH+1] = "";
	char brokerName[MAX_STRING_SIZE+1] = "";
	char executionGroupName[MAX_STRING_SIZE+1] = "";
	char messageFlowName[MAX_STRING_SIZE+1] = "";

	CciChar queueManagerNameUC[MQ_Q_MGR_NAME_LENGTH+1];
	CciChar brokerNameUC[MAX_STRING_SIZE+1];
	CciChar executionGroupNameUC[MAX_STRING_SIZE+1];
	CciChar messageFlowNameUC[MAX_STRING_SIZE+1];
	
	/* shared memory stuff */
	globalCB_t *sharedMemoryCBP_Broker = NULL,
		       *sharedMemoryCBP_Platform = NULL;
	char *sharedSegmentEnvironmentVariableName = "TIMER_SHARED_MEMORY_KSIZE";

	int numTimerCreateNodes = 0,
		numTimerCancelNodes = 0;
	int currentPID = 0;

	/* trace stuff */
	char *callersTraceEnvironmentVariable = "TIMER_PLUGIN_TRACE";
	int traceCallerIndex;
#else
	/* global variables */
    extern threadMutex_t timerChainMutex,
				         alarmThreadSyncMutex,
						 shmemThreadSyncMutex,
						 shmemThreadWorkMutex,
						 shmemInitTermMutex;
	extern threadCondVar_t alarmThreadWorkCondVar,
		                   alarmThreadSyncCondVar,
						   shmemThreadWorkCondVar,
						   shmemThreadSyncCondVar;
	extern bool_t shutdownAlarmRequested,
		          shutdownShmemRequested,
		          alarmThreadInitialised,
		          alarmThreadCreated,
		          shmemThreadInitialised,
		          shmemThreadCreated,
		          brokerInfoObtained,
				  alarmThreadShuttingDown,
				  shmemThreadShuttingDown,
		          firstTraceCall,
				  sharedMemoryAttached_Broker,
				  sharedMemoryAttached_Platform,
				  platformIsUnix,
				  sharedMemoryGarbageCollectorHasRun,
		          sharedMemoryExceptionThrown;

	extern deployedTimer_t timerAnchor,
		                  *timerAnchorP;
	extern deployedTimerCancel_t timerCancelAnchor,
		                         *timerCancelAnchorP;

	extern timerInstance_t instanceAnchor,
						   *nameIndexAnchorP,
						   *msgIdIndexAnchorP,
						   *correlIdIndexAnchorP,
						   *bothIndexAnchorP;
	extern messageFlow_t messageFlowAnchor;
	extern char *alarmThreadErrorMessage,
		        *shmemThreadErrorMessage;
	extern threadHandle_t alarmThreadHandle,
		                  shmemThreadHandle;

	extern char queueManagerName[];
	extern char brokerName[];
	extern char executionGroupName[];
	extern char messageFlowName[];

	extern CciChar queueManagerNameUC[];
	extern CciChar brokerNameUC[];
	extern CciChar executionGroupNameUC[];
	extern CciChar messageFlowNameUC[];
	
	extern globalCB_t *sharedMemoryCBP_Broker,
		              *sharedMemoryCBP_Platform;
	extern char *sharedSegmentEnvironmentVariableName;

	extern int numTimerCreateNodes,
		       numTimerApplyNodes;
	extern int currentPID;

	/* trace stuff */
	extern char *callersTraceEnvironmentVariable;
	extern int traceCallerIndex;
#endif

/* add to double-linked list after current element */
#define INSERT_AFTER_IN_ALARM_CHAIN(__currentElementP, __newElementP) \
__newElementP->nextAlarmP = __currentElementP->nextAlarmP; \
__newElementP->prevAlarmP = __currentElementP; \
if (__currentElementP->nextAlarmP) __currentElementP->nextAlarmP->prevAlarmP = __newElementP; \
__currentElementP->nextAlarmP = __newElementP; 

#define INSERT_AFTER_IN_MSG_ID_CHAIN(__currentElementP, __newElementP) \
__newElementP->nextMsgIdP = __currentElementP->nextMsgIdP; \
__newElementP->prevMsgIdP = __currentElementP; \
if (__currentElementP->nextMsgIdP) __currentElementP->nextMsgIdP->prevMsgIdP = __newElementP; \
__currentElementP->nextMsgIdP = __newElementP; 

#define INSERT_AFTER_IN_CORREL_ID_CHAIN(__currentElementP, __newElementP) \
__newElementP->nextCorrelIdP = __currentElementP->nextCorrelIdP; \
__newElementP->prevCorrelIdP = __currentElementP; \
if (__currentElementP->nextCorrelIdP) __currentElementP->nextCorrelIdP->prevCorrelIdP = __newElementP; \
__currentElementP->nextCorrelIdP = __newElementP; 

#define INSERT_AFTER_IN_BOTH_CHAIN(__currentElementP, __newElementP) \
__newElementP->nextBothP = __currentElementP->nextBothP; \
__newElementP->prevBothP = __currentElementP; \
if (__currentElementP->nextBothP) __currentElementP->nextBothP->prevBothP = __newElementP; \
__currentElementP->nextBothP = __newElementP; 

#define INSERT_AFTER_IN_NAME_CHAIN(__currentElementP, __newElementP) \
__newElementP->nextNameP = __currentElementP->nextNameP; \
__newElementP->prevNameP = __currentElementP; \
if (__currentElementP->nextNameP) __currentElementP->nextNameP->prevNameP = __newElementP; \
__currentElementP->nextNameP = __newElementP; 



/* remove from double-linked list */
#define REMOVE_FROM_LINKED_LIST(__currentElementP) \
__currentElementP->prevP->nextP = __currentElementP->nextP; \
if (__currentElementP->nextP) \
	__currentElementP->nextP->prevP = __currentElementP->prevP;

#define REMOVE_FROM_ALARM_CHAIN(__currentElementP) \
__currentElementP->prevAlarmP->nextAlarmP = __currentElementP->nextAlarmP; \
if (__currentElementP->nextAlarmP) \
	__currentElementP->nextAlarmP->prevAlarmP = __currentElementP->prevAlarmP;

#define REMOVE_FROM_MSG_ID_CHAIN(__currentElementP) \
__currentElementP->prevMsgIdP->nextMsgIdP = __currentElementP->nextMsgIdP; \
if (__currentElementP->nextMsgIdP) \
	__currentElementP->nextMsgIdP->prevMsgIdP = __currentElementP->prevMsgIdP;

#define REMOVE_FROM_CORREL_ID_CHAIN(__currentElementP) \
__currentElementP->prevCorrelIdP->nextCorrelIdP = __currentElementP->nextCorrelIdP; \
if (__currentElementP->nextCorrelIdP) \
	__currentElementP->nextCorrelIdP->prevCorrelIdP = __currentElementP->prevCorrelIdP;

#define REMOVE_FROM_BOTH_CHAIN(__currentElementP) \
__currentElementP->prevBothP->nextBothP = __currentElementP->nextBothP; \
if (__currentElementP->nextBothP) \
	__currentElementP->nextBothP->prevBothP = __currentElementP->prevBothP;

#define REMOVE_FROM_NAME_CHAIN(__currentElementP) \
__currentElementP->prevNameP->nextNameP = __currentElementP->nextNameP; \
if (__currentElementP->nextNameP) \
	__currentElementP->nextNameP->prevNameP = __currentElementP->prevNameP;

/* macros which manipulate the global chains */

#define INSERT_AFTER_IN_GLOBAL_TIMER_MSG_ID_CHAIN(__currentElementP, __newElementP, __sharedMemorySegment) \
__newElementP->nextGlobalMsgIdP = __currentElementP->nextGlobalMsgIdP; \
__newElementP->prevGlobalMsgIdP = SHM_ADDRESS2HANDLE(__currentElementP, __sharedMemorySegment); \
if (__currentElementP->nextGlobalMsgIdP != SHM_NULL_HANDLE) SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalMsgIdP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalMsgIdP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); \
__currentElementP->nextGlobalMsgIdP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); 

#define INSERT_AFTER_IN_GLOBAL_TIMER_CORREL_ID_CHAIN(__currentElementP, __newElementP, __sharedMemorySegment) \
__newElementP->nextGlobalCorrelIdP = __currentElementP->nextGlobalCorrelIdP; \
__newElementP->prevGlobalCorrelIdP = SHM_ADDRESS2HANDLE(__currentElementP, __sharedMemorySegment); \
if (__currentElementP->nextGlobalCorrelIdP != SHM_NULL_HANDLE) SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalCorrelIdP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalCorrelIdP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); \
__currentElementP->nextGlobalCorrelIdP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); 

#define INSERT_AFTER_IN_GLOBAL_TIMER_BOTH_CHAIN(__currentElementP, __newElementP, __sharedMemorySegment) \
__newElementP->nextGlobalBothP = __currentElementP->nextGlobalBothP; \
__newElementP->prevGlobalBothP = SHM_ADDRESS2HANDLE(__currentElementP, __sharedMemorySegment); \
if (__currentElementP->nextGlobalBothP != SHM_NULL_HANDLE) SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalBothP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalBothP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); \
__currentElementP->nextGlobalBothP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); 

#define INSERT_AFTER_IN_GLOBAL_TIMER_NAME_CHAIN(__currentElementP, __newElementP, __sharedMemorySegment) \
__newElementP->nextGlobalNameP = __currentElementP->nextGlobalNameP; \
__newElementP->prevGlobalNameP = SHM_ADDRESS2HANDLE(__currentElementP, __sharedMemorySegment); \
if (__currentElementP->nextGlobalNameP != SHM_NULL_HANDLE) SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalNameP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalNameP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); \
__currentElementP->nextGlobalNameP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); 

#define INSERT_AFTER_IN_GLOBAL_TIMER_CHAIN(__currentElementP, __newElementP, __sharedMemorySegment) \
__newElementP->nextGlobalP = __currentElementP->nextGlobalP; \
__newElementP->prevGlobalP = SHM_ADDRESS2HANDLE(__currentElementP, __sharedMemorySegment); \
if (__currentElementP->nextGlobalP != SHM_NULL_HANDLE) SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); \
__currentElementP->nextGlobalP = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); 

#define INSERT_AFTER_IN_GLOBAL_EG_NAME_CHAIN(__currentElementP, __newElementP, __sharedMemorySegment) \
__newElementP->nextEntry = __currentElementP->nextEntry; \
__newElementP->prevEntry = SHM_ADDRESS2HANDLE(__currentElementP, __sharedMemorySegment); \
if (__currentElementP->nextEntry != SHM_NULL_HANDLE) SHM_HANDLE2ADDRESS(__currentElementP->nextEntry, executionGroupEntry_t *, __sharedMemorySegment)->prevEntry = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); \
__currentElementP->nextEntry = SHM_ADDRESS2HANDLE(__newElementP, __sharedMemorySegment); 


#define REMOVE_FROM_GLOBAL_TIMER_MSG_ID_CHAIN(__currentElementP, __sharedMemorySegment) \
SHM_HANDLE2ADDRESS(__currentElementP->prevGlobalMsgIdP, globalTimerInstance_t *, __sharedMemorySegment)->nextGlobalMsgIdP = __currentElementP->nextGlobalMsgIdP; \
if (__currentElementP->nextGlobalMsgIdP != SHM_NULL_HANDLE) \
	SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalMsgIdP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalMsgIdP = __currentElementP->prevGlobalMsgIdP;

#define REMOVE_FROM_GLOBAL_TIMER_CORREL_ID_CHAIN(__currentElementP, __sharedMemorySegment) \
SHM_HANDLE2ADDRESS(__currentElementP->prevGlobalCorrelIdP, globalTimerInstance_t *, __sharedMemorySegment)->nextGlobalCorrelIdP = __currentElementP->nextGlobalCorrelIdP; \
if (__currentElementP->nextGlobalCorrelIdP != SHM_NULL_HANDLE) \
	SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalCorrelIdP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalCorrelIdP = __currentElementP->prevGlobalCorrelIdP;

#define REMOVE_FROM_GLOBAL_TIMER_BOTH_CHAIN(__currentElementP, __sharedMemorySegment) \
SHM_HANDLE2ADDRESS(__currentElementP->prevGlobalBothP, globalTimerInstance_t *, __sharedMemorySegment)->nextGlobalBothP = __currentElementP->nextGlobalBothP; \
if (__currentElementP->nextGlobalBothP != SHM_NULL_HANDLE) \
	SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalBothP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalBothP = __currentElementP->prevGlobalBothP;

#define REMOVE_FROM_GLOBAL_TIMER_NAME_CHAIN(__currentElementP, __sharedMemorySegment) \
SHM_HANDLE2ADDRESS(__currentElementP->prevGlobalNameP, globalTimerInstance_t *, __sharedMemorySegment)->nextGlobalNameP = __currentElementP->nextGlobalNameP; \
if (__currentElementP->nextGlobalNameP != SHM_NULL_HANDLE) \
	SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalNameP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalNameP = __currentElementP->prevGlobalNameP;

#define REMOVE_FROM_GLOBAL_TIMER_CHAIN(__currentElementP, __sharedMemorySegment) \
SHM_HANDLE2ADDRESS(__currentElementP->prevGlobalP, globalTimerInstance_t *, __sharedMemorySegment)->nextGlobalP = __currentElementP->nextGlobalP; \
if (__currentElementP->nextGlobalP != SHM_NULL_HANDLE) \
	SHM_HANDLE2ADDRESS(__currentElementP->nextGlobalP, globalTimerInstance_t *, __sharedMemorySegment)->prevGlobalP = __currentElementP->prevGlobalP;

#define REMOVE_FROM_GLOBAL_EG_NAME_CHAIN(__currentElementP, __sharedMemorySegment) \
SHM_HANDLE2ADDRESS(__currentElementP->prevEntry, executionGroupEntry_t *, __sharedMemorySegment)->nextEntry = __currentElementP->nextEntry; \
if (__currentElementP->nextEntry != SHM_NULL_HANDLE) \
	SHM_HANDLE2ADDRESS(__currentElementP->nextEntry, executionGroupEntry_t *, __sharedMemorySegment)->prevEntry = __currentElementP->prevEntry;



/* Function Prototypes */
int evaluateTimerCreate(context_t *contextP,
		                 CciMessage *destinationList,
		                 CciMessage *exceptionList,
		                 CciMessage *message);

int evaluateTimerCancel(context_t *contextP,
		                CciMessage *destinationList,
		                CciMessage *exceptionList,
		                CciMessage *message);

int createAlarmThread(IN context_t *contextP);

threadFunction_t processAlarms(void *dataP);

threadFunction_t sharedMemoryGarbageCollection(void *dataP);

void *getMessageIdElement(IN void *message);

void setTimerTimeoutAlarm(IN timerInstance_t *timerInstanceP);

int propagateMessage(IN context_t *contextP,
					 IN CciChar *terminalName,
					 IN void *message,
					 IN void *destinationList,
					 IN void *exceptionList,
                     IN bool_t newMessage);	

int processAttribute(IN context_t *contextP,
				     IN CciChar *attributeName,
				     IN CciChar *attributeValue);

void updateAllIndexes(IN timerInstance_t *newInstanceP,
					  IN scope_t scope,
					  IN bool_t removeMatchingInstances);

int searchIndexAndCancel(IN char *key,
				         IN indexType_t indexType,
				         IN scope_t scope,
				         OUT int *numMatchesP);

void freeTimer(IN timerInstance_t *timerInstanceP);

void unchainTimer(IN timerInstance_t *timerInstanceP);


int applyDataToElement(IN CciElement *elementP, 
					   IN timerInstance_t *timerP);

int generateTimerEventMessage(IN MQHCONN qmConn,
						      IN timerInstance_t *timerInstanceP);



#define CHECK_SHMEM_ERROR(__rc, __functionName, __scopeIsBroker, __ptr, __type, __size, __lockHeld, __isUnix, __retry) \
{                                                              \
	char msgBuffer[200];                                       \
    if (sharedMemoryExceptionThrown) {                         \
        TRACE_MID(TRACE_INFO, ("Shared memory exception already thrown - this thread will now exit")); \
		exit(0);                                               \
	}                                                          \
	if (__rc != TT_SUCCESS)                                    \
	{                                                          \
		if (__rc == TT_INSUFFICIENT_GLOBAL_STORAGE && __retry) \
		{                                                      \
		    /* poke the garbage collector to clean up any unused control blocks and try again */  \
	        TRACE_MID(TRACE_INFO, ("About to signal garbage collector thread")); \
			threadMutexLock(shmemThreadWorkMutex);             \
			sharedMemoryGarbageCollectorHasRun = FALSE;        \
			threadCondSignal(shmemThreadWorkCondVar);          \
			threadMutexUnlock(shmemThreadWorkMutex);           \
                                                               \
			while(!sharedMemoryGarbageCollectorHasRun)         \
			{                                                  \
		        TRACE_MID(TRACE_INFO, ("Waiting for shared memory garbage collector to run")); \
				SLEEP_THREAD_MILLISECONDS(1);                  \
			}                                                  \
                                                               \
			/* try allocating storage one more time */         \
			__ptr = (__type)shmMalloc(__size, __lockHeld, __scopeIsBroker, &__rc); \
		}                                                      \
		if (__rc == TT_INSUFFICIENT_GLOBAL_STORAGE)            \
		{                                                      \
			/* dump the heap */                                \
			/*                                                 \
		    shmDumpHeap("MikesHeapBigDump", __scopeIsBroker);  \
			*/                                                 \
			                                                   \
			/* raise exception */                              \
	        TRACE_MID(TRACE_ERROR, ("Insufficient global storage available - throwing exception")); \
			/* throw an exception here...*/                    \
		    sprintf(msgBuffer, "No available global storage"); \
		    sharedMemoryExceptionThrown = TRUE;                \
            if (__isUnix)                                      \
			{                                                  \
				cciThrowException(0,                               \
								  CCI_FATAL_EXCEPTION,             \
								  __FILE__,                        \
								  __LINE__,                        \
								  __functionName,                  \
								  C.PLUGIN_MESSAGE_CATALOGUE,      \
								  MJB_INSUFFICIENT_GLOBAL_STORAGE,  /* messageNumber */  \
								  msgBuffer, /* traceText */       \
								  brokerName,                      \
								  (char *)0);                      \
			}                                                      \
			else                                                   \
			{                                                      \
				cciThrowException(0,                               \
								  CCI_FATAL_EXCEPTION,             \
								  __FILE__,                        \
								  __LINE__,                        \
								  __functionName,                  \
								  C.PLUGIN_MESSAGE_CATALOGUE,      \
								  MJB_INSUFFICIENT_GLOBAL_STORAGE,  /* messageNumber */  \
								  msgBuffer, /* traceText */       \
								  (char *)0);                      \
			}                                                      \
        }                                                                  \
		if (__rc != TT_SUCCESS) {                                  \
		/* raise exception */                                  \
	    TRACE_MID(TRACE_ERROR, ("Error detected in shared memory")); \
		/* throw an exception here...*/                        \
		sprintf(msgBuffer, "Fatal error in shared memory function"); \
        sharedMemoryExceptionThrown = TRUE;                        \
        if (__isUnix)                                              \
		{                                                          \
			cciThrowException(0,                                   \
							  CCI_FATAL_EXCEPTION,                 \
							  __FILE__,                            \
							  __LINE__,                            \
							  __functionName,                      \
							  C.PLUGIN_MESSAGE_CATALOGUE,          \
							  MJB_FATAL_IN_GLOBAL_STORAGE,  /* messageNumber */  \
							  msgBuffer,              /* traceText */ \
							  brokerName,                          \
							  (char *)0);                          \
		}                                                          \
		else                                                       \
		{                                                          \
			cciThrowException(0,                                   \
							  CCI_FATAL_EXCEPTION,                 \
							  __FILE__,                            \
							  __LINE__,                            \
							  __functionName,                      \
							  C.PLUGIN_MESSAGE_CATALOGUE,          \
							  MJB_FATAL_IN_GLOBAL_STORAGE,  /* messageNumber */  \
							  msgBuffer,              /* traceText */ \
							  (char *)0);                          \
		}                                                          \
		}                                                          \
	}                                                              \
}

#define CHECK_SHMFREE_ERROR(__rc, __functionName, __isUnix) \
{                                                           \
	char msgBuffer[200];                                       \
    if (__rc >= TT_INVALID_EYECATCHER_ON_START && __rc <= TT_EYECATCHER_MISMATCH_ON_HEAP_2) { \
        if (sharedMemoryExceptionThrown) {                         \
            TRACE_MID(TRACE_INFO, ("Shared memory exception already thrown - this thread will now exit")); \
		    exit(0);                                               \
		}                                                           \
		/* log suitable message before shutting down process */               \
		switch (__rc) {                                                       \
		    case TT_INVALID_EYECATCHER_ON_START:                              \
	            sprintf(msgBuffer, "Eyecatcher at start of heap is corrupt"); \
			    break;                                                        \
		    case TT_INVALID_EYECATCHER_ON_END:                                \
				sprintf(msgBuffer, "Eyecatcher at end of heap is corrupt");   \
			    break;                                                        \
		    case TT_MUST_COMPLETE_STILL_SET:                                  \
				sprintf(msgBuffer, "'Must Complete' flag still set - possible heap corruption"); \
			    break;                                                        \
		    case TT_EYECATCHER_MISMATCH_ON_HEAP_1:                            \
				sprintf(msgBuffer, "Eyecatcher mismatch (1) on heap");        \
			    break;                                                        \
		    case TT_EYECATCHER_MISMATCH_ON_HEAP_2:                            \
				sprintf(msgBuffer, "Eyecatcher mismatch (2) on heap");        \
			    break;                                                        \
			default:                                                          \
			    ; /* do nothing */                                            \
		}                                                                     \
		/* raise exception */                                  \
	    TRACE_MID(TRACE_ERROR, ("Error detected in shared memory")); \
	    sharedMemoryExceptionThrown = TRUE;                          \
		/* throw an exception here...*/                            \
        if (__isUnix) {                                            \
			cciThrowException(0,                                   \
							  CCI_FATAL_EXCEPTION,                 \
							  __FILE__,                            \
							  __LINE__,                            \
							  __functionName,                      \
							  C.PLUGIN_MESSAGE_CATALOGUE,          \
							  MJB_FATAL_IN_FREE_GLOBAL_STORAGE,  /* messageNumber */  \
							  msgBuffer,              /* traceText */ \
							  brokerName,                          \
							  msgBuffer,              /* INSERT */ \
							  (char *)0);                          \
		}                                                          \
		else {                                                     \
			cciThrowException(0,                                   \
							  CCI_FATAL_EXCEPTION,                 \
							  __FILE__,                            \
							  __LINE__,                            \
							  __functionName,                      \
							  C.PLUGIN_MESSAGE_CATALOGUE,          \
							  MJB_FATAL_IN_FREE_GLOBAL_STORAGE,  /* messageNumber */  \
							  msgBuffer,              /* traceText */ \
							  msgBuffer,              /* INSERT */ \
							  (char *)0);                          \
		}                                                          \
	}                                                              \
}




#define RETHROW_IF_EXCEPTION(__rc) \
if (__rc == TT_PROPAGATION_EXCEPTION) \
		{ \
	/* about to re-throw last exception, reset the trace indentation first */ \
	TRACE_MID(TRACE_INFO, ("About to rethrow exception, resetting trace indentation")); \
	TraceSetIndentationValue(0); \
	cciRethrowLastException(0); \
}





#endif

