#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <cmqc.h>
#include <time.h>

#define NUM_MESSAGES 1
#define EVENT_WAIT_SECONDS 10

#define WAIT_FOR_NUM_EVENTS 2

#define XML_MESSAGE \
"<outer>"\
     "<tag1>data1</tag1>"\
     "<tag2>data2</tag2>"\
     "<tag3>data3</tag3>"\
	 "<inner>"\
	     "<innerMost>"\
             "<tag5>someTimerData</tag5>"\
	     "</innerMost>"\
	 "</inner>"\
	 "<interval>5</interval>"\
     "<tag4>data4</tag4>"\
"</outer>"

#define XML_MESSAGE_ALT \
"<outer tag1='data1'>"\
     "<tag2>data2</tag2>"\
     "<tag3>data3</tag3>"\
	 "<inner>"\
	     "<innerMost>"\
             "<tag5>someTimerData</tag5>"\
	     "</innerMost>"\
	 "</inner>"\
     "<tag4>data4</tag4>"\
"</outer>"

#define CREATE_Q "TIMER_CREATE.Q"
#define EVENT_Q "TIMER_EVENT.Q"
#define CANCEL_Q "TIMER_CANCEL.Q"

#if defined(WIN32)
    #define QM_DEFAULT "NT_QM"
#elif defined(_AIX)
    #define QM_DEFAULT "SEAIX_QM"
#elif defined(_SOLARIS)
    #define QM_DEFAULT "SUN_QM"
#elif defined(_HPUX)
    #define QM_DEFAULT "HP_QM"
#endif

int main(int argc, char *argv[])
{
	MQHCONN hConn;
	MQHOBJ createHandle, 
		   eventHandle,
		   cancelHandle;
    MQLONG   openOptions;               /* MQOPEN options                */
    MQOD od = {MQOD_DEFAULT};           /* Object Descriptor */
    MQPMO pmo = {MQPMO_DEFAULT};        /* put message options */
    MQGMO gmo = {MQGMO_DEFAULT};        /* put message options */
	MQMD md,
		 mdDefault = {MQMD_DEFAULT};           /* Message Descriptor */
	MQLONG compCode,
		   reason;
	int msgFlow2 = 0;
	int i,
		messageLength,
		returnedMessageLength;
        char correlId[MQ_CORREL_ID_LENGTH];
	char message[10000],
		 *qmName,
		 *eventQueue,
		 *cancelQueue,
		 *createQueue;

	/* display usage template if requested */
    if (argc == 2 && argv[1][0] == '?' ||
        argc > 1 && argc != 5)
	{
		printf("Usage: %s < <queueManagerName> <timerCreateQueue> <timerCancelQueue> <timerEventQueue>\n",
			    argv[0]);
		return(0);
	}

    if (argc > 1)
    {
        /* read in queue manager name */
		qmName = argv[1];

		/* now read the queue name(s) */
		createQueue = argv[2];
		cancelQueue = argv[3];
		eventQueue = argv[4];
    }
    else
    {
		qmName = QM_DEFAULT;
		createQueue = CREATE_Q;
		cancelQueue = CANCEL_Q;
		eventQueue = EVENT_Q;
    }

	printf("Queue manager: %s\n", qmName);
	printf("Create queue: %s\n", createQueue);
	printf("Cancel queue: %s\n", cancelQueue);
	printf("Event queue: %s\n", eventQueue);

	/* connect to queue manager, assume default */
	MQCONN(qmName,
		   &hConn,
		   &compCode,
		   &reason);
	if (reason != MQRC_NONE)
	{
		printf("MQCONN failed with compcode=%d, reason=%d\n",
			   compCode, reason);
		exit(1);
	}

	/* open the timer create queue */
	strcpy(od.ObjectName, createQueue);
	openOptions = MQOO_OUTPUT + MQOO_FAIL_IF_QUIESCING; /* open queue for output */

	MQOPEN(hConn,
		   &od,
		   openOptions,
		   &createHandle,
		   &compCode,
		   &reason);
	if (reason != MQRC_NONE)
	{
		printf("MQOPEN for input queue '%s' failed with compcode=%d, reason=%d\n",
			   od.ObjectName, compCode, reason);
		exit(1);
	}

	/* open the timer cancel queue */
	strcpy(od.ObjectName, cancelQueue);
	openOptions = MQOO_OUTPUT + MQOO_FAIL_IF_QUIESCING; /* open queue for output */

	MQOPEN(hConn,
		   &od,
		   openOptions,
		   &cancelHandle,
		   &compCode,
		   &reason);
	if (reason != MQRC_NONE)
	{
		printf("MQOPEN for input queue '%s' failed with compcode=%d, reason=%d\n",
			   od.ObjectName, compCode, reason);
		exit(1);
	}

	/* open the timer event queue */
	strcpy(od.ObjectName, eventQueue);
	openOptions = MQOO_INPUT_AS_Q_DEF + MQOO_FAIL_IF_QUIESCING; /* open queue for output */

	MQOPEN(hConn,
		   &od,
		   openOptions,
		   &eventHandle,
		   &compCode,
		   &reason);
	if (reason != MQRC_NONE)
	{
		printf("MQOPEN for timer event queue '%s' failed with compcode=%d, reason=%d\n",
			   od.ObjectName, compCode, reason);
		exit(1);
	}

	/* build BLOB type message or XML message */
#if 0
	sprintf(message, "This is a BLOB message");
#else
	strcpy(message, XML_MESSAGE);
#endif
	printf("Putting request message '%s' on queue %s\n",
			message, CREATE_Q);
	messageLength = strlen(message);
	memcpy(&md, &mdDefault, sizeof(md));

	/* put the message on the queue */
	MQPUT(hConn,                   /* connection handle */
		  createHandle,            /* object handle */
		  &md,                     /* message descriptor */
		  &pmo,                    /* put options */
		  (MQLONG)messageLength,   /* msg length */
		  message,                 /* message buffer */
		  &compCode,               /* completion code */
		  &reason);                /* reason code */
	if (reason != MQRC_NONE)
	{
		printf("MQPUT failed with compcode=%d, reason=%d\n",
			   compCode, reason);
		exit(1);
	}
	memcpy(correlId, md.MsgId, MQ_MSG_ID_LENGTH);


	/* read the required number of event messages from the event queue */
	for (i=0; i<WAIT_FOR_NUM_EVENTS; i++)
	{

		/* read message with matching correl id */
		fprintf(stderr, "about to read event %d from queue %s\n", i+1, EVENT_Q);
		messageLength = sizeof(message);
		memcpy(&md, &mdDefault, sizeof(md));
		memcpy(md.CorrelId, correlId, MQ_CORREL_ID_LENGTH);

		gmo.Options = MQGMO_WAIT +
		              MQGMO_NO_SYNCPOINT;
        gmo.WaitInterval = MQWI_UNLIMITED;

		/* get the message from the queue */
		MQGET(hConn,                   /* connection handle */
			  eventHandle,             /* object handle */
			  &md,                     /* message descriptor */
			  &gmo,                    /* put options */
			  (MQLONG)messageLength,   /* msg length */
			  message,                 /* message buffer */
	  (long *)&returnedMessageLength,  /* returned message length */
			  &compCode,               /* completion code */
			  &reason);                /* reason code */
		if (reason != MQRC_NONE)
		{
			printf("MQGET failed with compcode=%d, reason=%d\n",
				   compCode, reason);
			exit(1);
		}

		fprintf(stderr, "Timer event message is:\n'%.*s'\n",
			            returnedMessageLength, message);
	}

	/* now cancel the timer... */
#if 0
	sprintf(message, "This is a BLOB message");
#else
	strcpy(message, XML_MESSAGE);
#endif
	printf("Putting timer cancel message on queue %s\n",
			CANCEL_Q);
	messageLength = strlen(message);
	
	memcpy(&md, &mdDefault, sizeof(md));
	memcpy(md.CorrelId, correlId, MQ_CORREL_ID_LENGTH);

	/* put the message on the queue */
	MQPUT(hConn,                   /* connection handle */
		  cancelHandle,            /* object handle */
		  &md,                     /* message descriptor */
		  &pmo,                    /* put options */
		  (MQLONG)messageLength,   /* msg length */
		  message,                 /* message buffer */
		  &compCode,               /* completion code */
		  &reason);                /* reason code */
	if (reason != MQRC_NONE)
	{
		printf("MQPUT failed with compcode=%d, reason=%d\n",
			   compCode, reason);
		exit(1);
	}


	/* Now, with timer cancelled, try reading another event message - should fail */
	fprintf(stderr, "about to read event from queue %s (wait up to %d seconds)\n", 
		            EVENT_Q, EVENT_WAIT_SECONDS);
	messageLength = sizeof(message);
	memcpy(&md, &mdDefault, sizeof(md));
	memcpy(md.CorrelId, correlId, MQ_CORREL_ID_LENGTH);

	gmo.Options = MQGMO_WAIT +
		          MQGMO_NO_SYNCPOINT;
    gmo.WaitInterval = EVENT_WAIT_SECONDS * 1000;   /* wait up to 30 seconds for event message */

	/* get the message from the queue */
	MQGET(hConn,                   /* connection handle */
		  eventHandle,             /* object handle */
		  &md,                     /* message descriptor */
		  &gmo,                    /* put options */
		  (MQLONG)messageLength,   /* msg length */
		  message,                 /* message buffer */
  (long *)&returnedMessageLength,  /* returned message length */
		  &compCode,               /* completion code */
		  &reason);                /* reason code */
	if (reason != MQRC_NONE)
	{
		if (reason == MQRC_NO_MSG_AVAILABLE)
		{
			printf("SUCCESS - No message available on event queue\n");
		}
		else
		{
			printf("MQGET failed with compcode=%d, reason=%d\n",
				   compCode, reason);
			exit(1);
		}
	}
	else
	{
		fprintf(stderr, "FAILED - Timer event message received is:\n'%.*s'\n",
						returnedMessageLength, message);
	}


	MQDISC(&hConn,
		   &compCode,
		   &reason);
	if (reason != MQRC_NONE)
	{
		printf("MQDISC failed with compcode=%d, reason=%d\n",
			   compCode, reason);
		exit(1);
	}

    return(0);
}
