#if !defined(__TIMER_CONSTANTS_H)
    #define __TIMER_CONSTANTS_H
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Program name    : timer_constants.h                                        */
/*                                                                            */
/* Description     : This header file contains unicode constants for the      */
/*                   timer plug-in node.                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: Jul 2001                     */
/*                                                                            */
/******************************************************************************/

#include <BipCci.h>	  /* Data types for Plug-in Node functions     */

#if defined(WIN32)  // little endian, so no need to shift bytes 
	#define UC(__char) ((CciChar)__char)
	#define UCB ((CciChar)' ')
	#define UCDOT ((CciChar)'.')
#elif defined(_SOLARIS) || defined(_HPUX)
	#define UC(__char) (((CciChar)__char))
	#define UCB (((CciChar)' '))
	#define UCDOT (((CciChar)'.'))
#else
	#define UC(__char) (((CciChar)__char)<<8)
	#define UCB (((CciChar)' ')<<8)
	#define UCDOT (((CciChar)'.')<<8)
#endif

typedef struct timerConstants {
    CciChar 
        *PLUGIN_MESSAGE_CATALOGUE,
        *TIMER_CREATE_NODE,
        *TIMER_CANCEL_NODE,
        *ATTR_TIMER_NAME,
        *ATTR_TIMER_NAME_DEFAULT_VALUE,
        *ATTR_MESSAGE_DATA,
        *ATTR_MESSAGE_DATA_DEFAULT_VALUE,
        *ATTR_TIMER_QUEUE,
        *ATTR_MESSAGE_DATA_TYPE,
        *ATTR_MESSAGE_DATA_TYPE_DEFAULT_VALUE,
        *ATTR_MATCH_ON,
        *ATTR_MATCH_ON_DEFAULT_VALUE,
        *ATTR_INTERVAL_SECONDS,
        *ATTR_INTERVAL_SECONDS_DEFAULT_VALUE,
        *ATTR_TIMER_POP_COUNT,
        *ATTR_TIMER_POP_COUNT_DEFAULT_VALUE,
        *ATTR_TIMER_MSG_EXPIRY,
        *ATTR_TIMER_MSG_EXPIRY_DEFAULT_VALUE,
        *ATTR_TIMER_SCOPE,
        *ATTR_TIMER_SCOPE_DEFAULT_VALUE,
        *MATCH_CORRELID_TO_MSGID,
        *MATCH_MSGID_TO_MSGID,
        *MATCH_CORRELID_TO_CORRELID,
        *MATCH_MSGID_TO_CORRELID,
        *MATCH_BOTHIDS,
        *SCOPE_VALUE_EG,
        *SCOPE_VALUE_BROKER,
        *TIMER_THREAD_ID,
        *ATTR_LABEL,
        *ATTR_USER_TRACE_LEVEL,
        *ATTR_TRACE_LEVEL,
        *ATTR_USER_TRACE_FILTER,
        *ATTR_TRACE_FILTER,
        *TERM_IN,
        *TERM_OUT,
        *TERM_FAILURE,
        *TERM_NO_MATCH,
        *ROOT,
        *ROOT_DOT,
        *ENVIRONMENT,
        *BLOB,
        *NONE,
        *XML,
        *FORMAT,
        *NULL_STRING,
        *NULL_LITERAL,
        *TRUE_C,
        *FALSE_C,
        *DESTINATION,
        *MQ_DESTINATION_LIST,
        *DESTINATION_DATA,
        *MQMD,
        *MSG_ID,
        *CORREL_ID,
        *PUT_APPL_NAME,
        *REPLY_TO_Q,
        *REPLY_TO_Q_MGR,
        *QUEUE_MANAGER_NAME,
        *QUEUE_NAME,
        *ALL_NUMBERS,
        *TIMER_PLUGIN_FACTORY,
        DOT_CHAR,
        BLANK_CHAR;
} timerConstants_t;

/* build static initialisation values for Solaris */
#if defined(_SOLARIS) || defined(_HPUX)

#if defined(GLOBAL_TIMER_DATA)
CciChar 
    PLUGIN_MESSAGE_CATALOGUE_VAL[sizeof("MQSIV2_timer")*sizeof(CciChar)] =
    {'M','Q','S','I','V','2','_','t','i','m','e','r',0},
    TIMER_CREATE_NODE_VAL[sizeof("TimerCreateNode")*sizeof(CciChar)] =
    {'T','i','m','e','r','C','r','e','a','t','e','N','o','d','e',0},
    TIMER_CANCEL_NODE_VAL[sizeof("TimerCancelNode")*sizeof(CciChar)] =
    {'T','i','m','e','r','C','a','n','c','e','l','N','o','d','e',0},
    ATTR_TIMER_NAME_VAL[sizeof("TimerName")*sizeof(CciChar)] =
    {'T','i','m','e','r','N','a','m','e',0},
    ATTR_TIMER_NAME_DEFAULT_VALUE_VAL[sizeof("")*sizeof(CciChar)] =
    {0},
    ATTR_MESSAGE_DATA_VAL[sizeof("MessageData")*sizeof(CciChar)] =
    {'M','e','s','s','a','g','e','D','a','t','a',0},
    ATTR_MESSAGE_DATA_DEFAULT_VALUE_VAL[sizeof("")*sizeof(CciChar)] =
    {0},
    ATTR_TIMER_QUEUE_VAL[sizeof("TimerQueue")*sizeof(CciChar)] =
    {'T','i','m','e','r','Q','u','e','u','e',0},
    ATTR_MESSAGE_DATA_TYPE_VAL[sizeof("MessageDataType")*sizeof(CciChar)] =
    {'M','e','s','s','a','g','e','D','a','t','a','T','y','p','e',0},
    ATTR_MESSAGE_DATA_TYPE_DEFAULT_VALUE_VAL[sizeof("BLOB")*sizeof(CciChar)] =
    {'B','L','O','B',0},
    ATTR_MATCH_ON_VAL[sizeof("MatchOn")*sizeof(CciChar)] =
    {'M','a','t','c','h','O','n',0},
    ATTR_MATCH_ON_DEFAULT_VALUE_VAL[sizeof("CorrelId=OldMsgId")*sizeof(CciChar)] =
    {'C','o','r','r','e','l','I','d','=','O','l','d','M','s','g','I','d',0},
    ATTR_INTERVAL_SECONDS_VAL[sizeof("IntervalSeconds")*sizeof(CciChar)] =
    {'I','n','t','e','r','v','a','l','S','e','c','o','n','d','s',0},
    ATTR_INTERVAL_SECONDS_DEFAULT_VALUE_VAL[sizeof("15")*sizeof(CciChar)] =
    {'1','5',0},
    ATTR_TIMER_POP_COUNT_VAL[sizeof("TimerPopCount")*sizeof(CciChar)] =
    {'T','i','m','e','r','P','o','p','C','o','u','n','t',0},
    ATTR_TIMER_POP_COUNT_DEFAULT_VALUE_VAL[sizeof("1")*sizeof(CciChar)] =
    {'1',0},
    ATTR_TIMER_MSG_EXPIRY_VAL[sizeof("MessageExpiry")*sizeof(CciChar)] =
    {'M','e','s','s','a','g','e','E','x','p','i','r','y',0},
    ATTR_TIMER_MSG_EXPIRY_DEFAULT_VALUE_VAL[sizeof("0")*sizeof(CciChar)] =
    {'0',0},
    ATTR_TIMER_SCOPE_VAL[sizeof("TimerScope")*sizeof(CciChar)] =
    {'T','i','m','e','r','S','c','o','p','e',0},
    ATTR_TIMER_SCOPE_DEFAULT_VALUE_VAL[sizeof("ExecutionGroup")*sizeof(CciChar)] =
    {'E','x','e','c','u','t','i','o','n','G','r','o','u','p',0},
    MATCH_CORRELID_TO_MSGID_VAL[sizeof("CorrelId=OldMsgId")*sizeof(CciChar)] =
    {'C','o','r','r','e','l','I','d','=','O','l','d','M','s','g','I','d',0},
    MATCH_MSGID_TO_MSGID_VAL[sizeof("MsgId=OldMsgId")*sizeof(CciChar)] =
    {'M','s','g','I','d','=','O','l','d','M','s','g','I','d',0},
    MATCH_CORRELID_TO_CORRELID_VAL[sizeof("CorrelId=OldCorrelId")*sizeof(CciChar)] =
    {'C','o','r','r','e','l','I','d','=','O','l','d','C','o','r','r','e','l','I','d',0},
    MATCH_MSGID_TO_CORRELID_VAL[sizeof("MsgId=OldCorrelId")*sizeof(CciChar)] =
    {'M','s','g','I','d','=','O','l','d','C','o','r','r','e','l','I','d',0},
    MATCH_BOTHIDS_VAL[sizeof("MsgId+CorrelId=OldMsgId+OldCorrelId")*sizeof(CciChar)] =
    {'M','s','g','I','d','+','C','o','r','r','e','l','I','d','=','O','l','d','M','s','g','I','d','+','O','l','d','C','o','r','r','e','l','I','d',0},
    SCOPE_VALUE_EG_VAL[sizeof("ExecutionGroup")*sizeof(CciChar)] =
    {'E','x','e','c','u','t','i','o','n','G','r','o','u','p',0},
    SCOPE_VALUE_BROKER_VAL[sizeof("Broker")*sizeof(CciChar)] =
    {'B','r','o','k','e','r',0},
    TIMER_THREAD_ID_VAL[sizeof("TimerNodeTimerThread        ")*sizeof(CciChar)] =
    {'T','i','m','e','r','N','o','d','e','T','i','m','e','r','T','h','r','e','a','d',' ',' ',' ',' ',' ',' ',' ',' ',0},
    ATTR_LABEL_VAL[sizeof("label")*sizeof(CciChar)] =
    {'l','a','b','e','l',0},
    ATTR_USER_TRACE_LEVEL_VAL[sizeof("userTraceLevel")*sizeof(CciChar)] =
    {'u','s','e','r','T','r','a','c','e','L','e','v','e','l',0},
    ATTR_TRACE_LEVEL_VAL[sizeof("traceLevel")*sizeof(CciChar)] =
    {'t','r','a','c','e','L','e','v','e','l',0},
    ATTR_USER_TRACE_FILTER_VAL[sizeof("userTraceFilter")*sizeof(CciChar)] =
    {'u','s','e','r','T','r','a','c','e','F','i','l','t','e','r',0},
    ATTR_TRACE_FILTER_VAL[sizeof("traceFilter")*sizeof(CciChar)] =
    {'t','r','a','c','e','F','i','l','t','e','r',0},
    TERM_IN_VAL[sizeof("in")*sizeof(CciChar)] =
    {'i','n',0},
    TERM_OUT_VAL[sizeof("out")*sizeof(CciChar)] =
    {'o','u','t',0},
    TERM_FAILURE_VAL[sizeof("failure")*sizeof(CciChar)] =
    {'f','a','i','l','u','r','e',0},
    TERM_NO_MATCH_VAL[sizeof("noMatch")*sizeof(CciChar)] =
    {'n','o','M','a','t','c','h',0},
    ROOT_VAL[sizeof("Root")*sizeof(CciChar)] =
    {'R','o','o','t',0},
    ROOT_DOT_VAL[sizeof("Root.")*sizeof(CciChar)] =
    {'R','o','o','t','.',0},
    ENVIRONMENT_VAL[sizeof("Environment")*sizeof(CciChar)] =
    {'E','n','v','i','r','o','n','m','e','n','t',0},
    BLOB_VAL[sizeof("BLOB")*sizeof(CciChar)] =
    {'B','L','O','B',0},
    NONE_VAL[sizeof("none")*sizeof(CciChar)] =
    {'n','o','n','e',0},
    XML_VAL[sizeof("XML")*sizeof(CciChar)] =
    {'X','M','L',0},
    FORMAT_VAL[sizeof("Format")*sizeof(CciChar)] =
    {'F','o','r','m','a','t',0},
    NULL_STRING_VAL[sizeof("")*sizeof(CciChar)] =
    {0},
    NULL_LITERAL_VAL[sizeof("null")*sizeof(CciChar)] =
    {'n','u','l','l',0},
    TRUE_C_VAL[sizeof("true")*sizeof(CciChar)] =
    {'t','r','u','e',0},
    FALSE_C_VAL[sizeof("false")*sizeof(CciChar)] =
    {'f','a','l','s','e',0},
    DESTINATION_VAL[sizeof("Destination")*sizeof(CciChar)] =
    {'D','e','s','t','i','n','a','t','i','o','n',0},
    MQ_DESTINATION_LIST_VAL[sizeof("MQDestinationList")*sizeof(CciChar)] =
    {'M','Q','D','e','s','t','i','n','a','t','i','o','n','L','i','s','t',0},
    DESTINATION_DATA_VAL[sizeof("DestinationData")*sizeof(CciChar)] =
    {'D','e','s','t','i','n','a','t','i','o','n','D','a','t','a',0},
    MQMD_VAL[sizeof("MQMD")*sizeof(CciChar)] =
    {'M','Q','M','D',0},
    MSG_ID_VAL[sizeof("MsgId")*sizeof(CciChar)] =
    {'M','s','g','I','d',0},
    CORREL_ID_VAL[sizeof("CorrelId")*sizeof(CciChar)] =
    {'C','o','r','r','e','l','I','d',0},
    PUT_APPL_NAME_VAL[sizeof("PutApplName")*sizeof(CciChar)] =
    {'P','u','t','A','p','p','l','N','a','m','e',0},
    REPLY_TO_Q_VAL[sizeof("ReplyToQ")*sizeof(CciChar)] =
    {'R','e','p','l','y','T','o','Q',0},
    REPLY_TO_Q_MGR_VAL[sizeof("ReplyToQMgr")*sizeof(CciChar)] =
    {'R','e','p','l','y','T','o','Q','M','g','r',0},
    QUEUE_MANAGER_NAME_VAL[sizeof("queueManagerName")*sizeof(CciChar)] =
    {'q','u','e','u','e','M','a','n','a','g','e','r','N','a','m','e',0},
    QUEUE_NAME_VAL[sizeof("queueName")*sizeof(CciChar)] =
    {'q','u','e','u','e','N','a','m','e',0},
    ALL_NUMBERS_VAL[sizeof("0123456789")*sizeof(CciChar)] =
    {'0','1','2','3','4','5','6','7','8','9',0},
    TIMER_PLUGIN_FACTORY_VAL[sizeof("TimerPlugInFactory")*sizeof(CciChar)] =
    {'T','i','m','e','r','P','l','u','g','I','n','F','a','c','t','o','r','y',0};
#else
extern CciChar 
    PLUGIN_MESSAGE_CATALOGUE_VAL[],
    TIMER_CREATE_NODE_VAL[],
    TIMER_CANCEL_NODE_VAL[],
    ATTR_TIMER_NAME_VAL[],
    ATTR_TIMER_NAME_DEFAULT_VALUE_VAL[],
    ATTR_MESSAGE_DATA_VAL[],
    ATTR_MESSAGE_DATA_DEFAULT_VALUE_VAL[],
    ATTR_TIMER_QUEUE_VAL[],
    ATTR_MESSAGE_DATA_TYPE_VAL[],
    ATTR_MESSAGE_DATA_TYPE_DEFAULT_VALUE_VAL[],
    ATTR_MATCH_ON_VAL[],
    ATTR_MATCH_ON_DEFAULT_VALUE_VAL[],
    ATTR_INTERVAL_SECONDS_VAL[],
    ATTR_INTERVAL_SECONDS_DEFAULT_VALUE_VAL[],
    ATTR_TIMER_POP_COUNT_VAL[],
    ATTR_TIMER_POP_COUNT_DEFAULT_VALUE_VAL[],
    ATTR_TIMER_MSG_EXPIRY[],
    ATTR_TIMER_MSG_EXPIRY_DEFAULT_VALUE_VAL[],
    ATTR_TIMER_SCOPE_VAL[],
    ATTR_TIMER_SCOPE_DEFAULT_VALUE_VAL[],
    MATCH_CORRELID_TO_MSGID_VAL[],
    MATCH_MSGID_TO_MSGID_VAL[],
    MATCH_CORRELID_TO_CORRELID_VAL[],
    MATCH_MSGID_TO_CORRELID_VAL[],
    MATCH_BOTHIDS_VAL[],
    SCOPE_VALUE_EG_VAL[],
    SCOPE_VALUE_BROKER_VAL[],
    TIMER_THREAD_ID_VAL[],
    ATTR_LABEL_VAL[],
    ATTR_USER_TRACE_LEVEL_VAL[],
    ATTR_TRACE_LEVEL_VAL[],
    ATTR_USER_TRACE_FILTER_VAL[],
    ATTR_TRACE_FILTER_VAL[],
    TERM_IN_VAL[],
    TERM_OUT_VAL[],
    TERM_FAILURE_VAL[],
    TERM_NO_MATCH_VAL[],
    ROOT_VAL[],
    ROOT_DOT_VAL[],
    ENVIRONMENT_VAL[],
    BLOB_VAL[],
    NONE_VAL[],
    XML_VAL[],
    FORMAT_VAL[],
    NULL_STRING_VAL[],
    NULL_LITERAL_VAL[],
    TRUE_C_VAL[],
    FALSE_C_VAL[],
    DESTINATION_VAL[],
    MQ_DESTINATION_LIST_VAL[],
    DESTINATION_DATA_VAL[],
    MQMD_VAL[],
    MSG_ID_VAL[],
    CORREL_ID_VAL[],
    PUT_APPL_NAME_VAL[],
    REPLY_TO_Q_VAL[],
    REPLY_TO_Q_MGR_VAL[],
    QUEUE_MANAGER_NAME_VAL[],
    QUEUE_NAME_VAL[],
    ALL_NUMBERS_VAL[],
    TIMER_PLUGIN_FACTORY_VAL[];
#endif
#endif


/* define the initialisation macro */
#if defined(WIN32) || defined(_AIX)
#define TIMER_CONSTANTS_DEFAULTS \
    L"MQSIV2_timer", \
    L"TimerCreateNode", \
    L"TimerCancelNode", \
    L"TimerName", \
    L"", \
    L"MessageData", \
    L"", \
    L"TimerQueue", \
    L"MessageDataType", \
    L"BLOB", \
    L"MatchOn", \
    L"CorrelId=OldMsgId", \
    L"IntervalSeconds", \
    L"15", \
    L"TimerPopCount", \
    L"1", \
    L"MessageExpiry", \
    L"0", \
    L"TimerScope", \
    L"ExecutionGroup", \
    L"CorrelId=OldMsgId", \
    L"MsgId=OldMsgId", \
    L"CorrelId=OldCorrelId", \
    L"MsgId=OldCorrelId", \
    L"MsgId+CorrelId=OldMsgId+OldCorrelId", \
    L"ExecutionGroup", \
    L"Broker", \
    L"TimerNodeTimerThread        ", \
    L"label", \
    L"userTraceLevel", \
    L"traceLevel", \
    L"userTraceFilter", \
    L"traceFilter", \
    L"in", \
    L"out", \
    L"failure", \
    L"noMatch", \
    L"Root", \
    L"Root.", \
    L"Environment", \
    L"BLOB", \
    L"none", \
    L"XML", \
    L"Format", \
    L"", \
    L"null", \
    L"true", \
    L"false", \
    L"Destination", \
    L"MQDestinationList", \
    L"DestinationData", \
    L"MQMD", \
    L"MsgId", \
    L"CorrelId", \
    L"PutApplName", \
    L"ReplyToQ", \
    L"ReplyToQMgr", \
    L"queueManagerName", \
    L"queueName", \
    L"0123456789", \
    L"TimerPlugInFactory", \
    L'.', \
    L' '


#elif defined(_SOLARIS) || defined(_HPUX)

#define TIMER_CONSTANTS_DEFAULTS \
    PLUGIN_MESSAGE_CATALOGUE_VAL, \
    TIMER_CREATE_NODE_VAL, \
    TIMER_CANCEL_NODE_VAL, \
    ATTR_TIMER_NAME_VAL, \
    ATTR_TIMER_NAME_DEFAULT_VALUE_VAL, \
    ATTR_MESSAGE_DATA_VAL, \
    ATTR_MESSAGE_DATA_DEFAULT_VALUE_VAL, \
    ATTR_TIMER_QUEUE_VAL, \
    ATTR_MESSAGE_DATA_TYPE_VAL, \
    ATTR_MESSAGE_DATA_TYPE_DEFAULT_VALUE_VAL, \
    ATTR_MATCH_ON_VAL, \
    ATTR_MATCH_ON_DEFAULT_VALUE_VAL, \
    ATTR_INTERVAL_SECONDS_VAL, \
    ATTR_INTERVAL_SECONDS_DEFAULT_VALUE_VAL, \
    ATTR_TIMER_POP_COUNT_VAL, \
    ATTR_TIMER_POP_COUNT_DEFAULT_VALUE_VAL, \
    ATTR_TIMER_MSG_EXPIRY_VAL, \
    ATTR_TIMER_MSG_EXPIRY_DEFAULT_VALUE_VAL, \
    ATTR_TIMER_SCOPE_VAL, \
    ATTR_TIMER_SCOPE_DEFAULT_VALUE_VAL, \
    MATCH_CORRELID_TO_MSGID_VAL, \
    MATCH_MSGID_TO_MSGID_VAL, \
    MATCH_CORRELID_TO_CORRELID_VAL, \
    MATCH_MSGID_TO_CORRELID_VAL, \
    MATCH_BOTHIDS_VAL, \
    SCOPE_VALUE_EG_VAL, \
    SCOPE_VALUE_BROKER_VAL, \
    TIMER_THREAD_ID_VAL, \
    ATTR_LABEL_VAL, \
    ATTR_USER_TRACE_LEVEL_VAL, \
    ATTR_TRACE_LEVEL_VAL, \
    ATTR_USER_TRACE_FILTER_VAL, \
    ATTR_TRACE_FILTER_VAL, \
    TERM_IN_VAL, \
    TERM_OUT_VAL, \
    TERM_FAILURE_VAL, \
    TERM_NO_MATCH_VAL, \
    ROOT_VAL, \
    ROOT_DOT_VAL, \
    ENVIRONMENT_VAL, \
    BLOB_VAL, \
    NONE_VAL, \
    XML_VAL, \
    FORMAT_VAL, \
    NULL_STRING_VAL, \
    NULL_LITERAL_VAL, \
    TRUE_C_VAL, \
    FALSE_C_VAL, \
    DESTINATION_VAL, \
    MQ_DESTINATION_LIST_VAL, \
    DESTINATION_DATA_VAL, \
    MQMD_VAL, \
    MSG_ID_VAL, \
    CORREL_ID_VAL, \
    PUT_APPL_NAME_VAL, \
    REPLY_TO_Q_VAL, \
    REPLY_TO_Q_MGR_VAL, \
    QUEUE_MANAGER_NAME_VAL, \
    QUEUE_NAME_VAL, \
    ALL_NUMBERS_VAL, \
    TIMER_PLUGIN_FACTORY_VAL, \
    '.', \
    ' '
#endif


/* finally, define the global initializer structure */
#if defined(GLOBAL_TIMER_DATA)
    timerConstants_t C = {TIMER_CONSTANTS_DEFAULTS};
#else
    extern timerConstants_t C;
#endif


#define TIMER_CREATE_NODE_MB "TimerCreateNode"
#define TIMER_CANCEL_NODE_MB "TimerCancelNode"
#define ATTR_TIMER_NAME_MB "TimerName"
#define ATTR_MESSAGE_DATA_MB "MessageData"
#define ATTR_INTERVAL_SECONDS_MB "IntervalSeconds"
#define TIMER_THREAD_ID_MB "TimerNodeTimerThread        "


#define NULL_24 \
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
#define NULL_48 \
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
#define NULL_48_AND_TRAILING_NULL \
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0

#endif
