#if !defined(__UNICODE_H)
#define __UNICODE_H
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Progarm name    : unicode.h                                                */
/*                                                                            */
/* Description     : This header file contains function prototypes for        */
/*                   unicode conversion functions                             */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: Jun 2000                     */
/*                                                                            */
/******************************************************************************/

#if defined(WIN32)  || defined(_AIX)
	#include <wchar.h>
#endif

#if defined(_AIX)
	#include <wstring.h>
#endif

#include <BipCos.h>
#include <BipCci.h>
#include <stdio.h>
#include <string.h>

#if !defined(IN)
#define IN
#endif
#if !defined(OUT)
#define OUT
#endif
#if !defined(INOUT)
#define INOUT
#endif

/* 
 * On both NT and AIX, the wide character functions manipulate Unicode strings
 * correctly, however on Solaris, this is not the case and we need to implement our own
 * Unicode functions.
 */
#if defined(WIN32) || defined(_AIX)
    #define _USE_WCHAR_FUNCTIONS 1
#elif defined(_SOLARIS) || defined(_HPUX)
	#if defined(_USE_WCHAR_FUNCTIONS)
		#undef _USE_WCHAR_FUNCTIONS
	#endif
#endif


#if defined(_USE_WCHAR_FUNCTIONS)
	static CciChar *numericMask = L"0123456789";

    #define UClen(__string) wcslen(__string)
	#define UCcpy(__string1, __string2) wcscpy(__string1, __string2)
	#define UCncpy(__string1, __string2, __numChars) wcsncpy(__string1, __string2, __numChars)
	#define UCcmp(__string1, __string2) wcscmp(__string1, __string2)
	#define UCncmp(__string1, __string2, __numChars) wcsncmp(__string1, __string2, __numChars)
	#define UCchr(__string, __character) wcschr(__string, __character)
	#define UCcat(__string1, __string2) wcscat(__string1, __string2)
	#define UCsprintf sprintf
    #define UCstr(__string1, __string2) wcsstr(__string1, __string2)
#else
	static CciChar numericMask[10] = 
	{(CciChar)'0',(CciChar)'1',(CciChar)'2',(CciChar)'3',(CciChar)'4',
	 (CciChar)'5',(CciChar)'6',(CciChar)'7',(CciChar)'8',(CciChar)'9'};

	unsigned int UClen(CciChar *string);
	CciChar *UCcpy(CciChar *target, CciChar *source);
	CciChar *UCncpy(CciChar *target, CciChar *source, int numChars);
	CciChar *UCchr(CciChar *source, CciChar character);
	CciChar *UCcat(CciChar *string1, CciChar *string2);
	int UCcmp(CciChar *string1, CciChar *string2);
	int UCncmp(CciChar *string1, CciChar *string2, int numChars);
	int UCsprintf(char *buffer, char *format, ...);
	CciChar *UCstr(CciChar *string1, CciChar *string2);
#endif

/* The string duplication function wcsdup appears to have re-entrancy problems on AIX so
 * we'll use our own implementation of this for both AIX and Solaris
 */
#if defined(WIN32)
    #define UCdup(__string) wcsdup(__string)
#elif defined(_AIX) || defined(_SOLARIS) || defined(_HPUX)
	CciChar *UCdup(CciChar *source);
#endif


/****************/
/* strToUC      */
/****************/
CciChar *strToUC(IN char *string,
                 IN int length,
                 OUT CciChar *ucString,
				 OUT int *numConvertedBytesP);

/****************/
/* UCToStr      */
/****************/
char *UCToStr(IN CciChar *ucString,
              OUT char *string,
              IN int lengthOfStringBuffer,
			  OUT int *numConvertedBytesP);

/****************/
/* intToUC      */
/****************/
#if defined(WIN32)
	/* assume radix 10 */
    #define intToUC(__intVal, __buf) _itow(__intVal, __buf, 10)
#elif defined(_AIX) || defined(_SOLARIS) || defined(_HPUX) 
    int intToUC(int intVal, CciChar *buffer);
#endif


/****************/
/* UCToInt      */
/****************/
#if defined(WIN32)
    #define UCToInt(__ucString) _wtoi(__ucString)
#elif defined(_AIX) 
    #define UCToInt(__ucString) watoi(__ucString)
#elif defined(_SOLARIS) 
    int UCToInt(IN CciChar *string);
#elif defined(_HPUX) 
    int UCToInt(IN CciChar *string);
#endif

/****************/
/* strToUpper   */
/****************/
#if defined(WIN32)
    #define strToUpper(__mcString) strupr(__mcString)
#elif defined(_AIX) || defined(_SOLARIS) || defined(_HPUX)
	char *upperCase(INOUT char *string);
	char *lowerCase(INOUT char *string);
    #define strToUpper(__mcString) upperCase(__mcString)
#endif

/****************/
/* UCisNumeric  */
/****************/
#if !defined(bool_t)
	#define bool_t int
#endif
bool_t UCisNumeric(IN CciChar *string);


/**************/
/* CciString  */
/**************/
CciChar *CciString(IN char *source);

#endif



