#if !defined(__UTILITIES_H)
    #define __UTILITIES_H
/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Program name    : utilities.h                                              */
/*                                                                            */
/* Description     : This header file contains function prototypes and        */
/*                   constants for utilities.c                                */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/* Author: Mike Brady                      Date: Jun 2001                     */
/*                                                                            */
/******************************************************************************/

/* hack to get build to work ok under NT 4.0 */
#if 0  /* no longer seems to be required??? */
	#if !defined(_WIN32_WINNT)
		/* hack to get build to work ok under NT 4.0 */
		#define _WIN32_WINNT 0x0400
	#endif
#endif

#if defined(WIN32)
	#include <windows.h>
	#include <winbase.h>
#endif

#include <wchar.h>
#include <BipCci.h>	  /* Data types for Plug-in Node functions     */

#include <trace_publ.h>
#include <plugin.h>

#if !defined(TRACE_ALLOC)
	#if defined(TRACE_STORAGE)
		#define TRACE_ALLOC(__key, __addr) \
			TRACE_MID(TRACE_INFO, ("STORAGE("__key"): alloc %.8x", __addr));
	#else
		#define TRACE_ALLOC(__key, __addr) 
	#endif
#endif

#if !defined(TRACE_FREE)
	#if defined(TRACE_STORAGE)
		#define TRACE_FREE(__key, __addr) \
			TRACE_MID(TRACE_INFO, ("STORAGE("__key"): free %.8x", __addr));
	#else
		#define TRACE_FREE(__key, __addr) 
	#endif
#endif

#if defined(__THREADED)
    #include <threads.h>
#endif

#include <cmqc.h>
#include "unicode.h"
#if defined(BUILD_AGGREGATOR_PLUGIN)
    #include "MQSIV2_aggregator_msg.h"
    #include "aggregator_constants.h"
#elif defined(BUILD_POSTIT_PLUGIN)
    #include "MQSIV2_postit_msg.h"
    #include "postit_constants.h"
#elif defined(BUILD_TIMER_PLUGIN)
    #include "MQSIV2_timer_msg.h"
    #include "timer_constants.h"
#elif defined(BUILD_UUIDGEN_PLUGIN)
    #include "MQSIV2_uuidgen_msg.h"
    #include "uuidgen_constants.h"
#elif defined(BUILD_GETSQLCODE_PLUGIN)
    #include "MQSIV2_getSQLCode_msg.h"
    #include "getSQLCode_constants.h"
#elif defined(BUILD_FILEINPUT_PLUGIN)
    #include "MQSIV2_fileInput_msg.h"
    #include "fileInput_constants.h"
#elif defined(BUILD_FLOWMON_PLUGIN)
    #include "MQSIV2_flowmon_msg.h"
    #include "flowmon_constants.h"
#endif
#include "rc.h"

extern char brokerName[];

/* constants */
#define MAX_TIMER_NAME_LEN         256
#if !defined(IN)
    #define IN
#endif
#if !defined(OUT)
    #define OUT
#endif
#if !defined(INOUT)
    #define INOUT
#endif
#if !defined(TRUE)
    #define TRUE 1
#endif
#if !defined(FALSE)
    #define FALSE 0
#endif
#if !defined(bool_t)
    #define bool_t int
#endif

#define ONE_KBYTE 1024

/*
<?xml version="1.0" encoding="UTF-8"?><DATA>values</DATA>

This results in a message structure that looks like this...
MQMD as usual
(0x1000010)XML = (
    (0x5000018)XML = (
        (0x6000011)='1.0'
        (0x6000012)='UTF-8'
    )
    (0x1000000)DATA = (
        (0x2000000)='value'
    )
)
*/

#define MJB_ELEMENT_TYPE_XML_BODY 0x1000010 // element type value for XML body element			
#define MJB_ELEMENT_TYPE_DTD_HEADER 0x05000018 // element type value for XML header element			
#define MJB_ELEMENT_TYPE_DTD_VERSION 0x06000011 // element type value for XML version element			
#define MJB_ELEMENT_TYPE_DTD_ENCODING 0x06000012 // element type value for XML encoding element			


/* define unicode constants for utlities.c */

typedef struct utilStringConstants {
	CciChar *DESTINATION,
	        *MQ_DESTINATION_LIST,
	        *DESTINATION_DATA,
	        *MQMD,
	        *MSG_ID,
	        *CORREL_ID,
	        *PUT_APPL_NAME,
	        *REPLY_TO_Q,
	        *REPLY_TO_Q_MGR,
	        *NULL_STRING,
	        *ROOT,
	        *ENVIRONMENT,
			*XML,
	        *BLOB,
	        *NONE,
	        *NULL_LITERAL,
			*FORMAT,
			DOT_CHAR;
} utilStringConstants_t;


/* build static initialisation values for Solaris */
#if defined(_SOLARIS) || defined(_HPUX)

#if defined(DECLARE_UTILITIES_DATA)
CciChar
		U_DESTINATION_VAL[sizeof("Destination")*sizeof(CciChar)] =
{'D','e','s','t','i','n','a','t','i','o','n',0},
		U_MQ_DESTINATION_LIST_VAL[sizeof("MQDestinationList")*sizeof(CciChar)] =
{'M','Q','D','e','s','t','i','n','a','t','i','o','n','L','i','s','t',0},
		U_DESTINATION_DATA_VAL[sizeof("DestinationData")*sizeof(CciChar)] =
{'D','e','s','t','i','n','a','t','i','o','n','D','a','t','a',0},
		U_MQMD_VAL[sizeof("MQMD")*sizeof(CciChar)] =
{'M','Q','M','D',0},
		U_MSG_ID_VAL[sizeof("MsgId")*sizeof(CciChar)] =
{'M','s','g','I','d',0},
		U_CORREL_ID_VAL[sizeof("CorrelId")*sizeof(CciChar)] =
{'C','o','r','r','e','l','I','d',0},
		U_PUT_APPL_NAME_VAL[sizeof("PutApplName")*sizeof(CciChar)] =
{'P','u','t','A','p','p','l','N','a','m','e',0},
		U_REPLY_TO_Q_VAL[sizeof("ReplyToQ")*sizeof(CciChar)] =
{'R','e','p','l','y','T','o','Q',0},
		U_REPLY_TO_Q_MGR_VAL[sizeof("ReplyToQMgr")*sizeof(CciChar)] =
{'R','e','p','l','y','T','o','Q','M','g','r',0},
		U_NULL_STRING_VAL[sizeof("")*sizeof(CciChar)] =
{0},
		U_ROOT_VAL[sizeof("Root")*sizeof(CciChar)] =
{'R','o','o','t',0},
		U_ENVIRONMENT_VAL[sizeof("Environment")*sizeof(CciChar)] =
{'E','n','v','i','r','o','n','m','e','n','t',0},
		U_XML_VAL[sizeof("XML")*sizeof(CciChar)] =
{'X','M','L',0},
		U_BLOB_VAL[sizeof("BLOB")*sizeof(CciChar)] =
{'B','L','O','B',0},
		U_NONE_VAL[sizeof("none")*sizeof(CciChar)] =
{'n','o','n','e',0},
		U_NULL_LITERAL_VAL[sizeof("null")*sizeof(CciChar)] =
{'n','u','l','l',0},
		U_FORMAT_VAL[sizeof("Format")*sizeof(CciChar)] =
{'F','o','r','m','a','t',0};

#else
extern CciChar 
			U_DESTINATION_VAL[],
			U_MQ_DESTINATION_LIST_VAL[],
			U_DESTINATION_DATA_VAL[],
			U_MQMD_VAL[],
			U_MSG_ID_VAL[],
			U_CORREL_ID_VAL[],
			U_PUT_APPL_NAME_VAL[],
			U_REPLY_TO_Q_VAL[],
			U_REPLY_TO_Q_MGR_VAL[],
			U_NULL_STRING_VAL[],
			U_ROOT_VAL[],
			U_ENVIRONMENT_VAL[],
			U_XML_VAL[],
			U_BLOB_VAL[],
			U_NONE_VAL[],
			U_NULL_LITERAL_VAL[],
			U_FORMAT_VAL[];

#endif
#endif


/* define the initialisation macro */
#if defined(WIN32) || defined(_AIX)
#define UTIL_STRING_CONSTANTS_DEFAULTS \
    L"Destination",                         \
    L"MQDestinationList",                         \
    L"DestinationData",                         \
    L"MQMD",                         \
    L"MsgId",                         \
    L"CorrelId",                         \
    L"PutApplName",                         \
    L"ReplyToQ",                         \
    L"ReplyToQMgr",                         \
    L"",                         \
    L"Root",                         \
    L"Environment",                         \
    L"XML",                         \
    L"BLOB",                         \
    L"none",                         \
	L"",    \
    L"Format",                         \
    L'.'


#elif defined(_SOLARIS) || defined(_HPUX)

#define UTIL_STRING_CONSTANTS_DEFAULTS   \
    U_DESTINATION_VAL, \
    U_MQ_DESTINATION_LIST_VAL, \
    U_DESTINATION_DATA_VAL, \
    U_MQMD_VAL, \
    U_MSG_ID_VAL, \
    U_CORREL_ID_VAL, \
    U_PUT_APPL_NAME_VAL, \
    U_REPLY_TO_Q_VAL, \
    U_REPLY_TO_Q_MGR_VAL, \
    U_NULL_STRING_VAL, \
    U_ROOT_VAL, \
	U_ENVIRONMENT_VAL, \
    U_XML_VAL, \
	U_BLOB_VAL, \
	U_NONE_VAL, \
    U_NULL_LITERAL_VAL, \
	U_FORMAT_VAL, \
	'.'
	
#endif


/* finally, define the global initializer structure */

#if defined(DECLARE_UTILITIES_DATA)
    utilStringConstants_t U = {UTIL_STRING_CONSTANTS_DEFAULTS}; 
#else
    extern utilStringConstants_t U; 
#endif


/* return sequence numbers */
#define RSN_001     1
#define RSN_002     2
#define RSN_003     3
#define RSN_004     4
#define RSN_005     5
#define RSN_006     6
#define RSN_007     7
#define RSN_008     8
#define RSN_009     9
#define RSN_010     10
#define RSN_011     11
#define RSN_012     12
#define RSN_013     13
#define RSN_014     14
#define RSN_015     15
#define RSN_016     16
#define RSN_017     17
#define RSN_018     18
#define RSN_019     19
#define RSN_020     20
#define RSN_021     21
#define RSN_022     22
#define RSN_023     23
#define RSN_024     24
#define RSN_025     25
#define RSN_026     26
#define RSN_027     27
#define RSN_028     28
#define RSN_029     29
#define RSN_030     30
#define RSN_031     31
#define RSN_032     32
#define RSN_033     33
#define RSN_034     34
#define RSN_035     35
#define RSN_036     36
#define RSN_037     37
#define RSN_038     38
#define RSN_039     39
#define RSN_040     40
#define RSN_041     41
#define RSN_042     42
#define RSN_043     43
#define RSN_044     44
#define RSN_045     45
#define RSN_046     46
#define RSN_047     47
#define RSN_048     48
#define RSN_049     49
#define RSN_050     50
#define RSN_051     51
#define RSN_052     52
#define RSN_053     53
#define RSN_054     54
#define RSN_055     55
#define RSN_056     56
#define RSN_057     57
#define RSN_058     58
#define RSN_059     59
#define RSN_060     60
#define RSN_061     61
#define RSN_062     62
#define RSN_063     63
#define RSN_064     64
#define RSN_065     65
#define RSN_066     66
#define RSN_067     67
#define RSN_068     68
#define RSN_069     69
#define RSN_070     70


typedef struct msgIdIndexElement {
	char msgId[MQ_MSG_ID_LENGTH];
	void *memberP;
	struct msgIdIndexElement *nextP,
		                     *prevP;
} msgIdIndexElement_t;
#define MESSAGE_ID_INDEX_DEFAULT "",   \
                                 NULL, \
                                 NULL, \
							     NULL

typedef struct messageFlow {
	CciChar *name;
    void *anchorP,
	     *flowInstanceP;
	struct messageFlow *nextP,
		               *prevP;
} messageFlow_t;
#define MESSAGE_FLOW_DEFAULT NULL,                         \
                             NULL,\
                             NULL,                        \
                             NULL,                        \
						     NULL


typedef enum
{
	MESSAGE_ID,
	CORREL_ID,
	REPLY_TO_QUEUE,
	REPLY_TO_QUEUE_MANAGER,
	MQMD_ELEMENT,
	BODY,
	DESTINATION,
	MQ_DESTINATION_LIST,
	DESTINATION_DATA,
	PUT_APPL_NAME
} matchType_t;

#define ELEMENT_HAS_A_NAME     0x00000001
#define ELEMENT_HAS_A_VALUE    0x00000002
#define ELEMENT_HAS_A_CHILD    0x00000004

typedef struct headerFixed {
	int headerLength,
		depth;
	int flagWord;
	CciElementType elementType;
	CciValueType valueType;
} headerFixed_t;

#if 0
typedef struct headerFixed {
	int headerLength,
		depth;
	bool_t containsNameValueChild,
	       containsValueChild,
		   hasNameValueSibling,
		   hasAName,
		   hasAValue;
	CciElementType elementType;
	CciValueType valueType;
} headerFixed_t;
#endif

typedef struct elementHeader {
	headerFixed_t fixedPart;
	CciChar name[1];
} elementHeader_t;


typedef enum
{
	ELEMENT_TYPE_NAME = 1,
	ELEMENT_TYPE_VALUE,
	ELEMENT_TYPE_NAME_VALUE
} elementType_t;

msgIdIndexElement_t *updateMsgIdIndex(IN char *messageId,
									  IN msgIdIndexElement_t *indexAnchorP);

msgIdIndexElement_t *findMsgIndexElement(IN char *id,
									     IN msgIdIndexElement_t *indexAnchorP);

messageFlow_t *findMessageFlow(IN CciChar *name,
							   IN messageFlow_t *msgFlowAnchorP);

void *getMessageElement(IN void *message,
					    IN matchType_t matchType);

void traceMessageBuffer(IN CciMessage *message);

int getMessageLength(IN CciMessage *message,
					 OUT int *lengthP);

int swapMessageReplyDetails(IN CciMessage *message, 
							IN CciChar *newReplyToQueue,
							IN CciChar *newReplyToQueueManager,
							OUT CciChar *oldReplyToQueue,
							IN int oldReplyToQueueLength,
							OUT CciChar *oldReplyToQueueManager,
							IN int oldReplyToQueueManagerLength);

int swapMessageIdAndCorrelId(INOUT CciMessage *message);

void dispHex(IN char *inputString, 
			 IN int inputStringlength, 
			 OUT char *hexString);

int copyMessage(IN CciMessage *newMsg, 
			    IN CciMessage *oldMsg);

int propagateMessage(IN context_t *contextP,
					 IN CciChar *terminalName,
					 IN void *message,
					 IN void *destinationList,
					 IN void *exceptionList,
					 IN bool_t newMessage);

void *findElementInTree(void *currentElementP,
				           CciChar *elementName);

void *findChild(void *currentElementP,
				   CciChar *elementName);

CciMessage *createNewMessage(IN CciMessage *message);

int getMessageCharArrayValue(IN void *message,
							 IN matchType_t matchType,
							 OUT CciChar *buffer,
							 INOUT int *bufferLengthP,
							 OUT void **elementPP);

CciElement *locateElementInMessage(IN CciMessage *message,
								   IN CciChar *elementName,
								   IN bool_t createElement,
	 							   IN elementType_t *elementTypeP,
								   OUT bool_t *newElementCreatedP);

CciElement *locateElementInTree(IN CciMessage *message,
								IN CciElement *currentElement,
								IN CciChar *elementNameAndPath,
								IN bool_t createElement,
								IN elementType_t *elementTypeP,
								IN bool_t firstElementUnderRoot,
								OUT bool_t *newElementCreatedP);

int serialiseSyntaxElements(IN CciElement *copyFromElement,
							IN bool_t storeAsXML,
							IN bool_t ignoreDTD,
							OUT char **outputBufferP,
							OUT int *lengthP);

int inflateSyntaxElement(IN CciElement **elementPP,
						 IN char *buffer,
						 IN int bufferLength,
						 INOUT char **currentPosP,
						 INOUT int *treeDepthP,
						 IN bool_t firstElement,
						 IN bool_t includeFirstElement);

int deflateSyntaxElement(IN CciElement *elementP,
						 IN bool_t storeAsXML,
						 IN char **bufferP,
						 INOUT char **currentPosP,
						 INOUT int *bufferLengthP,
						 INOUT int *treeDepthP);

int deflateAllSyntaxElements(IN CciElement *elementP,
							 IN bool_t storeAsXML,
							 IN bool_t ignoreDTD,
						     IN char **bufferP,
						     INOUT char **currentPosP,
						     INOUT int *bufferLengthP,
						     INOUT int *treeDepthP);

int buildXMLFromSerialisedTree(IN void *serialisedData,
							   IN int serialisedDataLength,
							   OUT char **XMLP,
							   OUT int *XMLLengthP);

int convertSyntaxElementsToXML(IN char **XMLP,
						 	   IN int *XMLLengthP,
							   IN char **currentXMLP,
						       IN char *buffer,
						       IN int bufferLength,
						       INOUT char **currentPosP,
							   IN char *endPos,
						       INOUT int *treeDepthP);

bool_t extendBufferIfRequired(IN int storageRequired,
							  INOUT char **bufferP,
						      INOUT char **currentPosP,
						      INOUT int *bufferLengthP);

CciElement *createChildElement(IN CciElement *parentElementP,
						       IN CciChar *elementName,
							   IN CciElementType elementType);

CciElement *createSiblingElement(IN CciElement *parentElementP,
						         IN CciChar *elementName,
							     IN CciElementType elementType);

int getSharedSegmentSize(IN char *environmentVariableName,
						 IN int defaultSize);

int retrieveElementValue(IN CciElement *elementP,
						 IN bool_t lookForChildValueElement,
						 OUT char **bufferP,
						 OUT int *bufferLengthP);

int getValueFromSyntaxTree(IN CciMessage *message,
						   IN CciChar *elementName,
						   IN bool_t lookForChildValueElement,
						   OUT char **valueP,
						   OUT int *valueLengthP);

int formatDataValueForXML(IN  CciElement *elementP,
	  				      INOUT char *buffer,
				 	      IN int bufferLength,
					      OUT int *dataLengthP);

bool_t messageBodyIsXML(IN CciMessage *message);

bool_t isElementDTD(IN CciElement *elementP);

int deleteMessageBody(INOUT CciMessage *message);

int setMessageFormat(IN CciMessage *message,
	                 IN CciChar *format);


/* add to double-linked list after current element */
#define INSERT_AFTER_IN_LINKED_LIST(__currentElementP, __newElementP) \
__newElementP->nextP = __currentElementP->nextP; \
__newElementP->prevP = __currentElementP; \
if (__currentElementP->nextP) __currentElementP->nextP->prevP = __newElementP; \
__currentElementP->nextP = __newElementP;

#if !defined(REMOVE_FROM_LINKED_LIST)
	#define REMOVE_FROM_LINKED_LIST(__currentElementP) \
	__currentElementP->prevP->nextP = __currentElementP->nextP; \
	if (__currentElementP->nextP) \
		__currentElementP->nextP->prevP = __currentElementP->prevP;
#endif

#endif

